<?php

/*
 * Reads a simple product map file, and let the user see
 * if a product name is mapped to a combined product name
 */
class ProductMapping {
	
	/**
	 * Comments starts with #
	 * 
	 * Key word is: "MAP: " connected with the combind name e.g. "MAP: Geoframe"
	 * Every line after this keyword, contains of a product name that will be mapped
	 * to this combind name.
	 * 
	 * Product names can use wildcars '?' to match any character or 
	 * '*' to match any substring (0 or more characters).
	 * 
	 */

	
	private $file;
	private $map;
	
	
	public function ProductMapping($filname = null) {
		
		if (is_null($filname)) {
			throw new Exception("We need a product map file..");
		}

		/*if (is_file($filname)) {
			throw new Exception("Could not find map file {$filname}");
		}*/
		
		$this->map = array();
		$this->file = $filname;
		
		$this->load();
	}
	
	private function load() {
		
		if (!file_exists($this->file)) return;
		
		// Parse the mapping file, to an easier format for looking up data..
		$parser = new Parser(file_get_contents($this->file),
			ParserSettings::create(ParserSettings::COMMENT_CHAR, '#')
			->set(ParserSettings::SKIP_COMMENTS, true)
			->set(ParserSettings::SKIP_BLANK_LINES, true)
		);
		
		$product = null;
		
		// For each line we have...
		while ($parser->hasMore()) {
			
			// Get next line
			$line = $parser->next();
			
			// Check if got the keyword
			if (substr(trim($line->toString()), 0, 4) == 'MAP:') {
				$map = $line->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_SEMI_COLON));				
				$product = trim($map->get(1)->toString());
				$this->map[$product] = array();
				continue;
			}
			
			// if we dont have a product name, we ontinue..
			if (is_null($product)) continue;

			
			// add product name to combined product name
			$this->map[$product][] = trim($line->toString());
		}
		
	}
	
	public function getParent($productName) {		
		foreach ($this->map as $combinedName => $values) {
			foreach ($values as $value) {
				$value = trim(strtolower($value));
				
				$matched = $this->getMatched($value,strtolower($productName));
				if ($matched) {
					return $combinedName;
				}
			}
		}
		return null;		
	}
	
	private function getMatched($value,$productName) {

		$asterisk = stristr($value, '*') === FALSE;
		$question = stristr($value, '?') === FALSE;
		$pattern = "";
		
		//If no wildcards entered and if the value is equal to productName
		if (trim($value) == trim($productName) && ($asterisk && $question))
			return 1;
		
		//If all inputs are ?
		$qq = 1;
		$idx = 0;
		$arr = array();
		$lastPosKey = "-1";
		for($i=0; $i < strlen($value); $i++) {
			if(substr($value,$i,1) != "?") {
				$qq = 0; $idx++;
			} else {
				$arr[$idx] = $arr[$idx] . substr($value,$i,1);
				if($i == strlen($value) - 1) $lastPosKey = $idx;
			}
		}
		if ($qq) {
			if (preg_match("/^.{".strlen($value)."}$/",$productName)) {
				return 1;
			}
		}
			
		$reservedRegEx = array(".","+","$");							//Array of Reserved RegEx characters
		$replacementRegEx = array("\.","\+","\$");						//Add slashes in reservedRegEx
		
		$value = str_replace($reservedRegEx,$replacementRegEx,$value);	//Clean value for patterns.
		
		//Replace ? by groups and create Regex
		foreach($arr as $key => $val) {
			$first = ($key == 0) ? "^" : "";
			$last = ($lastPosKey == $key) ? "$" : "";
			if($lastPosKey == $key) {
				$value = preg_replace("/\?{".strlen($val)."}/",$first . ".{".strlen($val)."}" . $last,$value);
			} else {
				$value = preg_replace("/\?{".strlen($val)."}/", $first . ".{".strlen($val)."}" . $last, $value,1);
			}	
		}

		$value = str_replace('?','.',$value);							// Replace ? by .
		$pattern = "/" . str_replace('*','.*',$value) . "/";			// Replace * by .*

		//Checked if matched
		if(preg_match($pattern,$productName))
			return 1;
		else
			return 0;
	}

}

?>